function y = BarrierOp(S, X, T, r, q, sigma, K, H, PutCall, BarType)

% Single-barrier option pricing.

% S = Spot price.
% X = Strike Price.
% T = Maturity in years.
% r = Risk free rate.
% q = Cost of carry.
% sigma = Volatility.
% K = Rebate.
% H = Barrier
% PutCall = 'P'ut or 'C'all
% BarType = 'UO' 'DO' 'UI' 'UO'


% Define the indicators eta and phi for the different flavors of options.
switch PutCall
    case 'C'
        switch BarType
            case 'DI'
                eta = 1;
				phi = 1;
            case 'UI'
                eta = -1;
				phi = 1;
            case 'DO'
                eta = 1;
				phi = 1;
            case 'UO'
                eta = -1;
				phi = 1;
        end
    case 'P'
        switch BarType
            case 'DI'
                eta = 1;
				phi = -1;
            case 'UI'
                eta = -1;
				phi = -1;
            case 'DO'
                eta = 1;
				phi = -1;
            case 'UO'
                eta = -1;
				phi = -1;
		end
end

% Calculate the parameter values and other settings.
mu = (q - sigma^2/2)/sigma^2;
lambda = sqrt(mu^2 + 2*r/sigma^2);
x1 = log(S/X)/sigma/sqrt(T) + (1 + mu)*sigma*sqrt(T);
x2 = log(S/H)/sigma/sqrt(T) + (1 + mu)*sigma*sqrt(T);
y1 = log(H^2/S/X)/sigma/sqrt(T) + (1 + mu)*sigma*sqrt(T);
y2 = log(H/S)/sigma/sqrt(T) + (1 + mu)*sigma*sqrt(T);
Z = log(H/S)/sigma/sqrt(T) + lambda*sigma*sqrt(T);

% Calculate the components of the barrier option prices.
A = phi*S*exp((q - r)*T)*normcdf(phi*x1) - phi*X*exp(-r*T)*normcdf(phi*x1 - phi*sigma*sqrt(T));
B = phi*S*exp((q - r)*T)*normcdf(phi*x2) - phi*X*exp(-r*T)*normcdf(phi*x2 - phi*sigma*sqrt(T));
C = phi*S*exp((q - r)*T)*(H/S)^(2*(mu + 1))*normcdf(eta*y1) - phi*X*exp(-r*T)*(H/S)^(2*mu)*normcdf(eta*y1 - eta*sigma*sqrt(T));
D = phi*S*exp((q - r)*T)*(H/S)^(2*(mu + 1))*normcdf(eta*y2) - phi*X*exp(-r*T)*(H/S)^(2*mu)*normcdf(eta*y2 - eta*sigma*sqrt(T));
E = K*exp(-r*T)*(normcdf(eta*x2 - eta*sigma*sqrt(T)) - (H/S)^(2*mu)*normcdf(eta*y2 - eta*sigma*(sqrt(T))));
F = K*((H/S)^(mu + lambda)*normcdf(eta*Z) + (H/S)^(mu - lambda)*normcdf(eta*Z - 2*eta*lambda*sigma*sqrt(T)));

% Calculate the barrier option prices based on the option type.
switch PutCall
    case 'C'
        switch BarType
            case 'DI'
                if X > H 
                    y = C + E;
                else
                    y = A - B + D + E;
                end
            case 'UI'
                if X > H 
                    y = A + E;
                else
                    y = B - C + D + E;
                end
            case 'DO'
                if X > H 
                    y = A - C + F;
                else
                    y = B - D + F;
                end
            case 'UO'
                if X > H 
                    y = F;
                else
                    y = A - B + C - D + F;
                end
        end
    case 'P'
        switch BarType
            case 'DI'
                if X > H 
                    y = B - C + D + E;
                else
                    y = A + E;
                end
            case 'UI'
                if X > H 
                    y = A - B + D + E;
                else
                    y = C + E;
                end
            case 'DO'
                if X > H 
                    y = A - B + C - D + F;
                else
                    y = F;
                end
            case 'UO'
                if X > H 
                    y = B - D + F;
                else
                    y = A - C + F;
                end
        end
end

% If the option price is negative, return zero.
if y <= 0
	y = 0;
end
